@@ -24,6 +24,7 @@ type Basic struct {
 	Vdata       interface{}
 	VjsonInt    int
 	VjsonUint   uint
+	VjsonUint64 uint64
 	VjsonFloat  float64
 	VjsonNumber json.Number
 }
@@ -61,6 +62,11 @@ type EmbeddedSquash struct {
 	Vunique string
 }
 
+type EmbeddedPointerSquash struct {
+	*Basic  `mapstructure:",squash"`
+	Vunique string
+}
+
 type EmbeddedAndNamed struct {
 	Basic
 	Named   Basic
@@ -219,6 +225,7 @@ func TestBasicTypes(t *testing.T) {
 		"vdata":       42,
 		"vjsonInt":    json.Number("1234"),
 		"vjsonUint":   json.Number("1234"),
+		"vjsonUint64": json.Number("9223372036854775809"), // 2^63 + 1
 		"vjsonFloat":  json.Number("1234.5"),
 		"vjsonNumber": json.Number("1234.5"),
 	}
@@ -282,6 +289,10 @@ func TestBasicTypes(t *testing.T) {
 		t.Errorf("vjsonuint value should be 1234: %#v", result.VjsonUint)
 	}
 
+	if result.VjsonUint64 != 9223372036854775809 {
+		t.Errorf("vjsonuint64 value should be 9223372036854775809: %#v", result.VjsonUint64)
+	}
+
 	if result.VjsonFloat != 1234.5 {
 		t.Errorf("vjsonfloat value should be 1234.5: %#v", result.VjsonFloat)
 	}
@@ -655,6 +666,56 @@ func TestDecodeFrom_EmbeddedSquash(t *testing.T) {
 	}
 }
 
+func TestDecode_EmbeddedPointerSquash_FromStructToMap(t *testing.T) {
+	t.Parallel()
+
+	input := EmbeddedPointerSquash{
+		Basic: &Basic{
+			Vstring: "foo",
+		},
+		Vunique: "bar",
+	}
+
+	var result map[string]interface{}
+	err := Decode(input, &result)
+	if err != nil {
+		t.Fatalf("got an err: %s", err.Error())
+	}
+
+	if result["Vstring"] != "foo" {
+		t.Errorf("vstring value should be 'foo': %#v", result["Vstring"])
+	}
+
+	if result["Vunique"] != "bar" {
+		t.Errorf("vunique value should be 'bar': %#v", result["Vunique"])
+	}
+}
+
+func TestDecode_EmbeddedPointerSquash_FromMapToStruct(t *testing.T) {
+	t.Parallel()
+
+	input := map[string]interface{}{
+		"Vstring": "foo",
+		"Vunique": "bar",
+	}
+
+	result := EmbeddedPointerSquash{
+		Basic: &Basic{},
+	}
+	err := Decode(input, &result)
+	if err != nil {
+		t.Fatalf("got an err: %s", err.Error())
+	}
+
+	if result.Vstring != "foo" {
+		t.Errorf("vstring value should be 'foo': %#v", result.Vstring)
+	}
+
+	if result.Vunique != "bar" {
+		t.Errorf("vunique value should be 'bar': %#v", result.Vunique)
+	}
+}
+
 func TestDecode_EmbeddedSquashConfig(t *testing.T) {
 	t.Parallel()
 
@@ -1666,6 +1727,7 @@ func TestDecodeTable(t *testing.T) {
 				"Vdata":       []byte("data"),
 				"VjsonInt":    0,
 				"VjsonUint":   uint(0),
+				"VjsonUint64": uint64(0),
 				"VjsonFloat":  0.0,
 				"VjsonNumber": json.Number(""),
 			},
@@ -1707,6 +1769,7 @@ func TestDecodeTable(t *testing.T) {
 					"Vdata":       []byte("data"),
 					"VjsonInt":    0,
 					"VjsonUint":   uint(0),
+					"VjsonUint64": uint64(0),
 					"VjsonFloat":  0.0,
 					"VjsonNumber": json.Number(""),
 				},
@@ -2013,7 +2076,8 @@ func TestInvalidType(t *testing.T) {
 		t.Fatalf("error should be kind of Error, instead: %#v", err)
 	}
 
-	if derr.Errors[0] != "'Vstring' expected type 'string', got unconvertible type 'int'" {
+	if derr.Errors[0] !=
+		"'Vstring' expected type 'string', got unconvertible type 'int', value: '42'" {
 		t.Errorf("got unexpected error: %s", err)
 	}
 
@@ -2345,6 +2409,79 @@ func TestDecode_StructTaggedWithOmitempty_KeepNonEmptyValues(t *testing.T) {
 	}
 }
 
+func TestDecode_mapToStruct(t *testing.T) {
+	type Target struct {
+		String    string
+		StringPtr *string
+	}
+
+	expected := Target{
+		String: "hello",
+	}
+
+	var target Target
+	err := Decode(map[string]interface{}{
+		"string":    "hello",
+		"StringPtr": "goodbye",
+	}, &target)
+	if err != nil {
+		t.Fatalf("got error: %s", err)
+	}
+
+	// Pointers fail reflect test so do those manually
+	if target.StringPtr == nil || *target.StringPtr != "goodbye" {
+		t.Fatalf("bad: %#v", target)
+	}
+	target.StringPtr = nil
+
+	if !reflect.DeepEqual(target, expected) {
+		t.Fatalf("bad: %#v", target)
+	}
+}
+
+func TestDecoder_MatchName(t *testing.T) {
+	t.Parallel()
+
+	type Target struct {
+		FirstMatch  string `mapstructure:"first_match"`
+		SecondMatch string
+		NoMatch     string `mapstructure:"no_match"`
+	}
+
+	input := map[string]interface{}{
+		"first_match": "foo",
+		"SecondMatch": "bar",
+		"NO_MATCH":    "baz",
+	}
+
+	expected := Target{
+		FirstMatch:  "foo",
+		SecondMatch: "bar",
+	}
+
+	var actual Target
+	config := &DecoderConfig{
+		Result: &actual,
+		MatchName: func(mapKey, fieldName string) bool {
+			return mapKey == fieldName
+		},
+	}
+
+	decoder, err := NewDecoder(config)
+	if err != nil {
+		t.Fatalf("err: %s", err)
+	}
+
+	err = decoder.Decode(input)
+	if err != nil {
+		t.Fatalf("err: %s", err)
+	}
+
+	if !reflect.DeepEqual(expected, actual) {
+		t.Fatalf("Decode() expected: %#v, got: %#v", expected, actual)
+	}
+}
+
 func testSliceInput(t *testing.T, input map[string]interface{}, expected *Slice) {
 	var result Slice
 	err := Decode(input, &result)
